@@ -252,6 +252,7 @@ eog_image_init (EogImage *img)
 	priv->thumbnail = NULL;
 	priv->width = priv->height = -1;
 	priv->modified = FALSE;
+	priv->autorotate = FALSE;
 	priv->mutex = g_mutex_new ();
 	priv->status_mutex = g_mutex_new ();
 	priv->load_finished = NULL;
@@ -355,20 +356,24 @@ update_exif_data (EogImage *image)
 	
 	entry = exif_content_get_entry (priv->exif->ifd [EXIF_IFD_EXIF], EXIF_TAG_PIXEL_X_DIMENSION);
 	if (entry != NULL && (priv->width >= 0)) {
-		exif_set_long (entry->data, bo, priv->width);
+		if (entry->format == EXIF_FORMAT_LONG)
+			exif_set_long (entry->data, bo, priv->width);
+		else if (entry->format == EXIF_FORMAT_SHORT)
+			exif_set_short (entry->data, bo, priv->width);
+		else
+			g_warning ("Exif entry has unsupported size");
 	}
 	
 	entry = exif_content_get_entry (priv->exif->ifd [EXIF_IFD_EXIF], EXIF_TAG_PIXEL_Y_DIMENSION);
 	if (entry != NULL && (priv->height >= 0)) {
-		exif_set_long (entry->data, bo, priv->height);
+		if (entry->format == EXIF_FORMAT_LONG)
+			exif_set_long (entry->data, bo, priv->height);
+		else if (entry->format == EXIF_FORMAT_SHORT)
+			exif_set_short (entry->data, bo, priv->height);
+		else
+			g_warning ("Exif entry has unsupported size");
 	}
 }
-#else
-static void
-update_exif_data (EogImage *image)
-{
-	/* nothing todo if we don't have exif support */
-}
 #endif
 
 static void
@@ -794,6 +799,53 @@ eog_image_real_load (EogImage *img, guint data2read, EogJob *job, GError **error
 	return !failed;
 }
 
+static gint
+eog_image_get_exif_orientation (EogImage *img)
+{
+	gint orientation = 0;
+	EogImagePrivate *priv = NULL;
+
+	g_return_val_if_fail (EOG_IS_IMAGE (img), 0);
+
+	priv = img->priv;
+	  
+#ifdef HAVE_EXIF
+	if (priv->exif != NULL) {
+		ExifEntry *e = exif_data_get_entry (priv->exif, EXIF_TAG_ORIENTATION);
+
+		if (e != NULL && e->data != NULL) {
+			ExifByteOrder o = exif_data_get_byte_order (e->parent->parent);
+			orientation = exif_get_short (e->data, o);
+		}
+	}
+#endif /* HAVE_EXIF */
+
+	return orientation;
+}
+
+static void
+eog_image_real_autorotate (EogImage *img)
+{
+	gint orientation = 0;
+
+	g_return_if_fail (EOG_IS_IMAGE (img));
+
+	orientation = eog_image_get_exif_orientation (img);
+
+	EogTransformType type = eog_transform_convert_exif_orientation (orientation);
+
+	if (type != EOG_TRANSFORM_NONE) {
+		EogTransform *transform = eog_transform_new (type);
+
+		eog_image_transform (img, transform, NULL);
+
+		g_object_unref (transform);
+	}
+
+	/* Disable auto orientation for next loads */
+	img->priv->autorotate = FALSE;
+}
+
 gboolean
 eog_image_has_data (EogImage *img, guint req_data)
 {
@@ -887,6 +939,12 @@ eog_image_load (EogImage *img, guint data2read, EogJob *job, GError **error)
 		}
 	}
 
+#ifdef HAVE_EXIF
+	if (success && priv->autorotate) {
+		eog_image_real_autorotate (img);
+	}
+#endif
+
 	/* update status */
 	if (success) {
 		priv->status = EOG_IMAGE_STATUS_LOADED;
@@ -1583,7 +1641,8 @@ eog_image_free_mem_private (EogImage *image)
 
 #ifdef HAVE_LCMS
 		if (priv->profile != NULL) {
-		  cmsCloseProfile (priv->profile);
+			cmsCloseProfile (priv->profile);
+			priv->profile = NULL;
 		}
 #endif
 		
@@ -1777,3 +1836,19 @@ eog_image_data_unref (EogImage *img)
 	
 	return img;
 }
+
+void
+eog_image_autorotate (EogImage *img)
+{
+	guint orientation = 0;
+
+	g_return_if_fail (EOG_IS_IMAGE (img));
+
+	orientation = eog_image_get_exif_orientation (img);
+
+	if (img->priv->exif != NULL) {
+		eog_image_real_autorotate (img);
+	} else {
+		img->priv->autorotate = TRUE;
+	}
+}
